import java.awt.event.WindowAdapter;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.io.FileNotFoundException;
import java.io.PrintStream;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.filechooser.FileNameExtensionFilter;

public class DigitalCircuitFrame extends JFrame {
	public static final int DEFAULT_WIDTH=1024;
	public static final int DEFAULT_HEIGHT=800;
	private DigitalCircuit circuit;
	
	public DigitalCircuitFrame() {
		
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		setTitle("Digital Circuit Laboratory");
		setSize((int)screen.getWidth()*3/4, (int)screen.getHeight()*3/4);
		
		JSplitPane Spanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		Spanel.setDividerLocation(850);
		Spanel.setOneTouchExpandable(false);
		
		circuit = new DigitalCircuit(this);
		JPanel labPanel = new DigitalCircuitLabPanel(circuit);
		JTabbedPane tab = new JTabbedPane(); // adds a tab in JPanel
		ButtonPanel jp2 = new ButtonPanel(circuit,this);
		circuit.setPanel(labPanel);   // to repaint view when model changes
	    tab.addTab( "Simulation 1", null, labPanel, "Primer panel" );   
	    
		Spanel.add( tab );
		Spanel.add( jp2 );
		closeWindow();
		add(Spanel);  
		
		JMenuBar menuBar = createLabMenuBar(circuit);
		setJMenuBar(menuBar);
		setVisible(true);
	
	}

	public JMenuBar createLabMenuBar(DigitalCircuit circuit) {
		EditMenuListener editListener = new EditMenuListener(circuit,this);
		FileMenuListener fileListener = new FileMenuListener(circuit);
		SimulateMenuListener simulateListener = new SimulateMenuListener(circuit);

		JMenuBar mb = new JMenuBar();
		
		JMenu menuFile = new JMenu ("File");
		mb.add(menuFile);
		
		JMenuItem menuItem = new JMenuItem("Open");
		menuItem.addActionListener(fileListener);
		menuFile.add(menuItem);
		
		menuItem = new JMenuItem("Save");
		menuItem.addActionListener(fileListener);
		menuFile.add(menuItem);
		menuFile.addSeparator();
		
		menuItem = new JMenuItem("Exit");
		menuItem.addActionListener(fileListener);
		menuFile.add(menuItem);
		
		JMenu menuInsert = new JMenu ("Insert");
		mb.add(menuInsert);
		JMenu subMenu = new JMenu("Gates");  
		menuInsert.add(subMenu);

		menuItem = new JMenuItem("Inverter", new ImageIcon("images/inverter.png"));
		menuItem.addActionListener(editListener);
		subMenu.add(menuItem);
		
		menuItem = new JMenuItem("Or", new ImageIcon("images/or.png"));
		menuItem.addActionListener(editListener);
		subMenu.add(menuItem);		
		
		menuItem = new JMenuItem("Nor", new ImageIcon("images/nor.png"));
		menuItem.addActionListener(editListener);
		subMenu.add(menuItem);

		menuItem = new JMenuItem("And", new ImageIcon("images/and.png"));
		menuItem.addActionListener(editListener);
		subMenu.add(menuItem);
		
		menuItem = new JMenuItem("Nand", new ImageIcon("images/nand.png"));
		menuItem.addActionListener(editListener);
		subMenu.add(menuItem);		
		
		menuItem = new JMenuItem("Xor", new ImageIcon("images/nand.png"));
		menuItem.addActionListener(editListener);
		subMenu.add(menuItem);
		
		JMenu insSubMenu = new JMenu("Instruments");  
		menuInsert.add(insSubMenu);
		
		menuItem = new JMenuItem("Wire", new ImageIcon("images/wire.png"));
		menuItem.addActionListener(editListener);
		insSubMenu.add(menuItem);
		insSubMenu.addSeparator();
		
		menuItem = new JMenuItem("Source", new ImageIcon("images/source.png"));
		menuItem.addActionListener(editListener);
		insSubMenu.add(menuItem);
		
		menuItem = new JMenuItem("Meter", new ImageIcon("images/meter.png"));
		menuItem.addActionListener(editListener);
		insSubMenu.add(menuItem);

		JMenu configMenu = new JMenu("Configurations");
		mb.add(configMenu);
		
		menuItem = new JMenuItem("Delete all");
		menuItem.addActionListener(editListener);
		configMenu.add(menuItem);
		
		JMenu menuColor = new JMenu("Change Elements Colors");
		menuItem.addActionListener(editListener);
		configMenu.add(menuColor);
		
		menuItem = new JMenuItem("Selected Color");
		menuItem.addActionListener(editListener);
		menuColor.add(menuItem);
		
		menuItem = new JMenuItem("Release Color");
		menuItem.addActionListener(editListener);
		menuColor.add(menuItem);
		
		JMenu menuSim = new JMenu("Simulate");
		mb.add(menuSim);
		menuItem = new JMenuItem("Start", new ImageIcon("images/run.png"));
		menuItem.addActionListener(simulateListener);
		menuSim.add(menuItem);
		
		return mb;          
	}
	/**
	 * Generates a method to close the window.
	 */
	public void closeWindow(){
		//window listener with abstract class
		 addWindowListener(new WindowAdapter() {
			   public void windowClosing(WindowEvent e) {
				   // ask if i wanna close
				   int r1 = JOptionPane.showConfirmDialog(null,"Exit from Circuit Simulator ?","Exit",JOptionPane.YES_NO_OPTION);
					if (r1 == JOptionPane.YES_OPTION) {
						// ask if i wanna save the simulation
						r1 = JOptionPane.showConfirmDialog(null,"Do you want to save the simulation?","Save",JOptionPane.YES_NO_OPTION);
						
						if (r1 == JOptionPane.YES_OPTION){
							
						   	  PrintStream outData = null;
					    	  JFileChooser fc = new JFileChooser();
					    	  FileNameExtensionFilter fil = new FileNameExtensionFilter(".dc", "dc");
					    	  fc.setFileFilter(fil);

					    	  int returnV = fc.showSaveDialog(null);
					    	  if(returnV == JFileChooser.APPROVE_OPTION) {
					    		  try {
					    			  outData = new PrintStream(fc.getSelectedFile());
					    		  } catch(FileNotFoundException ef) {
					    			  JOptionPane.showMessageDialog(null, "Can´t save the file.\nPlease try again. ","Error", JOptionPane.ERROR_MESSAGE);

					    			  return;
					    		  }
					    		  circuit.addSaveFile(outData);
					    	  }
						}
						// close all the program
						System.exit(0);
					}
			   }
			 });
	}


}
