import java.awt.*;
import java.awt.geom.*;
import java.util.Scanner;
/**
 * Class for Source visual object.
 * Encapsulates color behaviors and movements.
 * @author Marcio B & Antonio M
 *
 */
public class GSource extends Source implements Drawable, GOutSignal {
   private static final Path2D.Double polyline = new Path2D.Double(Path2D.WIND_EVEN_ODD,5); 
   private static final Rectangle2D.Double output= new Rectangle2D.Double(55,5,10,10);

   static {  // static initialization block. 
      polyline.moveTo (55, 10);
      polyline.lineTo(40,10);
      polyline.lineTo(40,40);
      polyline.lineTo(10,40);
      polyline.lineTo(10,-20);
      polyline.lineTo(40,-20);
      polyline.lineTo(40,10);
      polyline.moveTo(20,-10);
      polyline.lineTo(25, 0);
      polyline.lineTo(30, -10);
      polyline.moveTo(18, 25);
      polyline.curveTo(18, 35, 25, 35, 25, 25);
      polyline.curveTo(25, 15, 32, 15, 32, 25);
   }
   
	private Color color = Color.BLUE;
	private Color selColor = Color.RED;
	private Color relColor = Color.BLUE;
   private Path2D.Double shape;
   private Path2D.Double out;
   private static BasicStroke stroke;
   private Simulator s;

   /**
    * Constructor for GSource class
    * @param pos Point2D.Double
    * @param in Scanner
    * @param sim Simulator
    */
   public GSource(Point2D.Double pos, Scanner in, Simulator sim){
	   super(in, sim);
	   s = sim;   
	   AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
	   shape = (Path2D.Double) at.createTransformedShape(polyline);
	   out = (Path2D.Double) at.createTransformedShape(output);
	   stroke = new BasicStroke(2);

   }
   public void draw (Graphics2D g){
      g.setColor(color);
      g.setStroke(stroke);
      g.draw(shape);
      g.draw(out);
      getId();
      g.drawString("SOURCE_"+ getId(), (float)(shape.getCurrentPoint().getX()-55), (float)shape.getCurrentPoint().getY()+35);
   }
   
   public boolean contains (Point p){
     return shape.contains(p);
   }
   /**
    * Moves the object Source.
    */
   public void dragIn(int dx, int dy) {
      AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
      shape.transform(at);
      out.transform(at);

   }
   /**
    * Flips the Source.
    */
   public void flip() {
	   AffineTransform af = new AffineTransform();   
	   Rectangle2D box = shape.getBounds2D(); 
	   double x = box.getX()+box.getWidth()/2;
	   // Note: the transformations get applied IN REVERSE ORDER!
	   af.translate(x,0);
	   af.scale(-1, 1);
	   af.translate(-x, 0);          
	   shape.transform(af);
	   out.transform(af);

   }
   /**
    * Set object to selected
    */
   public void setSelected() {
     color = selColor;
   }
   /**
    * Set object to released
    */
   public void setReleased(){
     color = relColor;
   }
   /**
    * Set color for selected
    */
   public void setSelColor(Color c){
	   selColor = c;
   }
   /**
    * Set color for released
    */
   public void setRelColor(Color c){
	   relColor = c;
   }

   public Shape getGOutput() {
     return out;
   }
   /**
    * Set delay..
    * Not usable
    */
   public void setGDelay(float d){
	   
   }
   
   public void readd() {
		// TODO Auto-generated method stub
	    ChangeEvent e = new ChangeEvent(this, true, s.getTime());
	    s.add(e);
		
	}
@Override
public Path2D.Double getPos() {
	// TODO Auto-generated method stub
	return shape;
}
@Override
public float getGDelay() {
	// TODO Auto-generated method stub
	return 0;
}
@Override
public int getIdDraw() {
	// TODO Auto-generated method stub
	return getId();
}
@Override
public void graph() {
	// TODO Auto-generated method stub
	
}
}