import java.util.*;
/**
 * Base Class for GNorGate
 * @author Marcio Baeza
 *
 */
public class NorGate extends Gate {
  public NorGate (float delay, Simulator s) {
    super(delay, nextId++, s);
    sources= new ArrayList<OutSignal>();
  }
  /**
   * Constructor for NorGate class
   * @param s Simulator
   */
  public NorGate (Simulator s) {
     this(NorGate.delay, s);
  }
  public int getId(){
	  return getIdGate();
  }
  
  /**
   * 
   * Connect the sources with the outputs
   * @param output OutSignal
   */
  public void connectInput(OutSignal output){
    sources.add(output);
    output.connectDrain(this);
  }
  /**
   * Disconnect the sources from the drains
   */
  public void disconnectInput(){
    while(sources.size() != 0) {
      sources.get(0).removeFromDrains(this);
      sources.remove(0);
    }
  }
  /**
   * Add a output to the sources
   * @param output OutSignal
   */
  public void addSource(OutSignal output) {
    sources.add(output);
  }
  /**
   * Remove output from sources
   * @param output OutSignal
   */
  public void removeFromSource(OutSignal output){
    sources.remove(output);
  }
  /**
   * Compute Output for the gate
   * @return boolean
   */
  public boolean computeOutput(){
    boolean result=false;
    for ( OutSignal in: sources)
       result = result || in.getValue(); 
    return !result;
  }
  /**
   * Returns if the objects has a source connected
   * @return ArrayList of OutSignals
   */
  public boolean hasConnectedSource() {
    return sources.size()>0;
  }
  private ArrayList<OutSignal> sources;
  private static int nextId = 0;
  private static int delay = 2;
}